@@ -23,10 +23,12 @@
 #include <errno.h>
 #include <getopt.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #include <time.h>
 
 #include <espeak-ng/espeak_ng.h>
@@ -47,9 +49,9 @@ extern ESPEAK_NG_API int GetFileLength(const char *filename);
 static const char *help_text =
     "\n" PROGRAM_NAME " [options] [\"<words>\"]\n\n"
     "-f <text file>   Text file to speak\n"
-    "--stdin    Read text input from stdin instead of a file\n\n"
-    "If neither -f nor --stdin, then <words> are spoken, or if none then text\n"
-    "is spoken from stdin, each line separately.\n\n"
+    "--stdin    Read text input from stdin at once till to the end of a stream.\n\n"
+    "If neither -f nor --stdin are provided, then <words> from arguments are spoken,\n"
+	"or text is spoken from stdin, read separately one line by line at a time.\n\n"
     "-a <integer>\n"
     "\t   Amplitude, 0 to 200, default is 100\n"
     "-d <device>\n"
@@ -112,10 +114,11 @@ static const char *help_text =
     "--voices=<language>\n"
     "\t   List the available voices for the specified language.\n"
     "\t   If <language> is omitted, then list all voices.\n"
+    "--load     Load voice from a file in current directory by name.\n"
     "-h, --help Show this help.\n";
 
-int samplerate;
-int quiet = 0;
+static int samplerate;
+bool quiet = false;
 unsigned int samples_total = 0;
 unsigned int samples_split = 0;
 unsigned int samples_split_seconds = 0;
@@ -125,7 +128,7 @@ FILE *f_wavfile = NULL;
 char filetype[5];
 char wavefile[200];
 
-void DisplayVoices(FILE *f_out, char *language)
+static void DisplayVoices(FILE *f_out, char *language)
 {
 	int ix;
 	const char *p;
@@ -196,7 +199,7 @@ static void Write4Bytes(FILE *f, int value)
 	}
 }
 
-int OpenWavFile(char *path, int rate)
+static int OpenWavFile(char *path, int rate)
 {
 	static unsigned char wave_hdr[44] = {
 		'R', 'I', 'F', 'F', 0x24, 0xf0, 0xff, 0x7f, 'W', 'A', 'V', 'E', 'f', 'm', 't', ' ',
@@ -320,6 +323,7 @@ int main(int argc, char **argv)
 		{ "compile-mbrola", optional_argument, 0, 0x10e },
 		{ "compile-intonations", no_argument, 0, 0x10f },
 		{ "compile-phonemes", optional_argument, 0, 0x110 },
+		{ "load",    no_argument,       0, 0x111 },
 		{ 0, 0, 0, 0 }
 	};
 
@@ -335,6 +339,7 @@ int main(int argc, char **argv)
 	int value;
 	int flag_stdin = 0;
 	int flag_compile = 0;
+	int flag_load = 0;
 	int filesize = 0;
 	int synth_flags = espeakCHARS_AUTO | espeakPHONEMES | espeakENDPAUSE;
 
@@ -348,7 +353,7 @@ int main(int argc, char **argv)
 	int phoneme_options = 0;
 	int option_linelength = 0;
 	int option_waveout = 0;
-
+	
 	espeak_VOICE voice_select;
 	char filename[200];
 	char voicename[40];
@@ -404,7 +409,7 @@ int main(int argc, char **argv)
 			pitch = atoi(optarg2);
 			break;
 		case 'q':
-			quiet = 1;
+			quiet = true;
 			break;
 		case 'f':
 			strncpy0(filename, optarg2, sizeof(filename));
@@ -440,10 +445,15 @@ int main(int argc, char **argv)
 			break;
 		case 0x101: // --compile-debug
 		case 0x102: // --compile
-			strncpy0(voicename, optarg2, sizeof(voicename));
-			flag_compile = c;
-			quiet = 1;
-			break;
+			if (optarg2 != NULL && *optarg2) {
+				strncpy0(voicename, optarg2, sizeof(voicename));
+				flag_compile = c;
+				quiet = true;
+				break;
+			} else {
+				fprintf(stderr, "Voice name to '%s' not specified.\n", c == 0x101 ? "--compile-debug" : "--compile");
+				exit(EXIT_FAILURE);
+			}
 		case 0x103: // --punct
 			option_punctuation = 1;
 			if (optarg2 != NULL) {
@@ -556,6 +566,9 @@ int main(int argc, char **argv)
 			}
 			return EXIT_SUCCESS;
 		}
+		case 0x111: // --load
+			flag_load = 1;
+			break;
 		default:
 			exit(0);
 		}
@@ -597,9 +610,12 @@ int main(int argc, char **argv)
 	}
 
 	if (voicename[0] == 0)
-		strcpy(voicename, "en");
+		strcpy(voicename, ESPEAKNG_DEFAULT_VOICE);
 
-	result = espeak_ng_SetVoiceByName(voicename);
+	if(flag_load)
+		result = espeak_ng_SetVoiceByFile(voicename);
+	else
+		result = espeak_ng_SetVoiceByName(voicename);
 	if (result != ENS_OK) {
 		memset(&voice_select, 0, sizeof(voice_select));
 		voice_select.languages = voicename;
@@ -674,7 +690,7 @@ int main(int argc, char **argv)
 		size = strlen(p_text);
 		espeak_Synth(p_text, size+1, 0, POS_CHARACTER, 0, synth_flags, NULL, NULL);
 	} else if (flag_stdin) {
-		int max = 1000;
+		size_t max = 1000;
 		if ((p_text = (char *)malloc(max)) == NULL) {
 			espeak_ng_PrintStatusCodeMessage(ENOMEM, stderr, NULL);
 			exit(EXIT_FAILURE);
@@ -685,6 +701,8 @@ int main(int argc, char **argv)
 			while (fgets(p_text, max, f_text) != NULL) {
 				p_text[max-1] = 0;
 				espeak_Synth(p_text, max, 0, POS_CHARACTER, 0, synth_flags, NULL, NULL);
+				// Allow subprocesses to use the audio data through pipes.
+				fflush(stdout);
 			}
 			if (f_text != stdin) {
 				fclose(f_text);
@@ -697,13 +715,17 @@ int main(int argc, char **argv)
 					break;
 				p_text[ix++] = (char)c;
 				if (ix >= (max-1)) {
-					max += 1000;
-					char *new_text = (char *)realloc(p_text, max);
+					char *new_text = NULL;
+					if (max <= SIZE_MAX - 1000) {
+						max += 1000;
+						new_text = (char *)realloc(p_text, max);
+					}
 					if (new_text == NULL) {
 						free(p_text);
 						espeak_ng_PrintStatusCodeMessage(ENOMEM, stderr, NULL);
 						exit(EXIT_FAILURE);
 					}
+					p_text = new_text;
 				}
 			}
 			if (ix > 0) {
@@ -711,6 +733,8 @@ int main(int argc, char **argv)
 				espeak_Synth(p_text, ix+1, 0, POS_CHARACTER, 0, synth_flags, NULL, NULL);
 			}
 		}
+
+		free(p_text);
 	} else if (f_text != NULL) {
 		if ((p_text = (char *)malloc(filesize+1)) == NULL) {
 			espeak_ng_PrintStatusCodeMessage(ENOMEM, stderr, NULL);
@@ -721,6 +745,8 @@ int main(int argc, char **argv)
 		p_text[filesize] = 0;
 		espeak_Synth(p_text, filesize+1, 0, POS_CHARACTER, 0, synth_flags, NULL, NULL);
 		fclose(f_text);
+
+		free(p_text);
 	}
 
 	result = espeak_ng_Synchronize();
