@@ -35,17 +35,18 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
-#include "readclause.h"
-#include "synthdata.h"
-#include "wavegen.h"
-
-#include "error.h"
-#include "phoneme.h"
-#include "voice.h"
-#include "synthesize.h"
-#include "spect.h"
-#include "translate.h"
-#include "dictionary.h"
+#include "dictionary.h"               // for strncpy0
+#include "error.h"                    // for create_file_error_context
+#include "mnemonics.h"               // for LookupMnemName, MNEM_TAB
+#include "phoneme.h"                  // for PHONEME_TAB, PHONEME_TAB_LIST
+#include "readclause.h"               // for Read4Bytes
+#include "spect.h"                    // for SpectFrame, peak_t, SpectSeq
+#include "speech.h"			// for path_home, GetFileLength
+#include "synthdata.h"                // for LoadPhData
+#include "synthesize.h"               // for TUNE, frame_t, CONDITION_IS_OTHER
+#include "translate.h"                // for utf8_out, utf8_in
+#include "voice.h"                    // for LoadVoice, voice
+#include "wavegen.h"                  // for WavegenInit, WavegenSetVoice
 
 #define N_ITEM_STRING 256
 
@@ -416,8 +417,6 @@ static bool after_if = false;
 
 static char current_fname[80];
 
-static int markers_used[8];
-
 typedef struct {
 	void *link;
 	int value;
@@ -428,11 +427,6 @@ typedef struct {
 
 static REF_HASH_TAB *ref_hash_tab[256];
 
-#define N_ENVELOPES  30
-int n_envelopes = 0;
-char envelope_paths[N_ENVELOPES][80];
-unsigned char envelope_dat[N_ENVELOPES][ENV_LEN];
-
 typedef struct {
 	FILE *file;
 	int linenum;
@@ -857,7 +851,7 @@ static int NextItemMax(int max)
 
 static int NextItemBrackets(int type, int control)
 {
-	// Expect a parameter inside parantheses
+	// Expect a parameter inside parentheses
 	// control: bit 0  0= need (
 	//          bit 1  1= allow comma
 
@@ -1339,7 +1333,7 @@ static int LoadWavefile(FILE *f, const char *fname)
 	return displ | 0x800000; // set bit 23 to indicate a wave file rather than a spectrum
 }
 
-static espeak_ng_STATUS LoadEnvelope(FILE *f, const char *fname, int *displ)
+static espeak_ng_STATUS LoadEnvelope(FILE *f, int *displ)
 {
 	char buf[128];
 
@@ -1353,12 +1347,6 @@ static espeak_ng_STATUS LoadEnvelope(FILE *f, const char *fname, int *displ)
 		return errno;
 	fwrite(buf, 128, 1, f_phdata);
 
-	if (n_envelopes < N_ENVELOPES) {
-		strncpy0(envelope_paths[n_envelopes], fname, sizeof(envelope_paths[0]));
-		memcpy(envelope_dat[n_envelopes], buf, sizeof(envelope_dat[0]));
-		n_envelopes++;
-	}
-
 	return ENS_OK;
 }
 
@@ -1379,14 +1367,13 @@ static int Hash8(const char *string)
 	return (hash+chars) & 0xff;
 }
 
-static int LoadEnvelope2(FILE *f, const char *fname)
+static int LoadEnvelope2(FILE *f)
 {
 	int ix, ix2;
 	int n;
 	int x, y;
 	int displ;
 	int n_points;
-	double yy;
 	char line_buf[128];
 	float env_x[20];
 	float env_y[20];
@@ -1412,17 +1399,16 @@ static int LoadEnvelope2(FILE *f, const char *fname)
 		env_y[n_points] = env_y[n_points-1];
 	}
 
-	ix = -1;
+	ix = 0;
 	ix2 = 0;
 	if (n_points > 0) for (x = 0; x < ENV_LEN; x++) {
-		if (n_points > 3 && x > env_x[ix+4])
+		if (n_points > 3 && x > env_x[ix+3])
 			ix++;
 		if (n_points > 2 && x >= env_x[ix2+1])
 			ix2++;
 
 		if (env_lin[ix2] > 0) {
-			yy = env_y[ix2] + (env_y[ix2+1] - env_y[ix2]) * ((float)x - env_x[ix2]) / (env_x[ix2+1] - env_x[ix2]);
-			y = (int)(yy * 2.55);
+			y = (env_y[ix2] + (env_y[ix2+1] - env_y[ix2]) * ((float)x - env_x[ix2]) / (env_x[ix2+1] - env_x[ix2])) * 2.55;
 		} else if (n_points > 3)
 			y = (int)(polint(&env_x[ix], &env_y[ix], 4, x) * 2.55); // convert to range 0-255
 		else
@@ -1432,12 +1418,6 @@ static int LoadEnvelope2(FILE *f, const char *fname)
 		env[x] = y;
 	}
 
-	if (n_envelopes < N_ENVELOPES) {
-		strncpy0(envelope_paths[n_envelopes], fname, sizeof(envelope_paths[0]));
-		memcpy(envelope_dat[n_envelopes], env, ENV_LEN);
-		n_envelopes++;
-	}
-
 	displ = ftell(f_phdata);
 	fwrite(env, 1, ENV_LEN, f_phdata);
 
@@ -1498,10 +1478,10 @@ static espeak_ng_STATUS LoadDataFile(const char *path, int control, int *addr)
 			*addr = LoadWavefile(f, path);
 			type_code = 'W';
 		} else if (id == 0x43544950) {
-			status = LoadEnvelope(f, path, addr);
+			status = LoadEnvelope(f, addr);
 			type_code = 'E';
 		} else if (id == 0x45564E45) {
-			*addr = LoadEnvelope2(f, path);
+			*addr = LoadEnvelope2(f);
 			type_code = 'E';
 		} else {
 			error("File not SPEC or RIFF: %s", path);
@@ -1852,7 +1832,7 @@ static PHONEME_TAB_LIST *FindPhonemeTable(const char *string)
 		if (strcmp(phoneme_tab_list2[ix].name, string) == 0)
 			return &phoneme_tab_list2[ix];
 	}
-	error("Unknown phoneme table: '%s'", string);
+	error("compile: unknown phoneme table: '%s'", string);
 	return NULL;
 }
 
@@ -2540,17 +2520,14 @@ espeak_ng_CompilePhonemeDataPath(long rate,
 
 	samplerate_native = samplerate = rate;
 	LoadPhData(NULL, NULL);
-	if (LoadVoice("", 0) == NULL)
+	if (LoadVoice("", 8/*compiling phonemes*/) == NULL)
 		return ENS_VOICE_NOT_FOUND;
 
 	WavegenInit(rate, 0);
 	WavegenSetVoice(voice);
 
-	n_envelopes = 0;
 	error_count = 0;
 	resample_count = 0;
-	memset(markers_used, 0, sizeof(markers_used));
-
 	f_errors = log;
 
 	strncpy0(current_fname, "phonemes", sizeof(current_fname));
@@ -2654,6 +2631,8 @@ espeak_ng_CompilePhonemeDataPath(long rate,
 
 	fclose(f_report);
 
+	WavegenFini();
+
 	if (resample_count > 0) {
 		fprintf(f_errors, "\n%d WAV files resampled to %d Hz\n", resample_count, samplerate_native);
 		fprintf(log, "Compiled phonemes: %d errors, %d files resampled to %d Hz.\n", error_count, resample_count, samplerate_native);
