@@ -34,15 +34,14 @@
 #include <espeak-ng/encoding.h>
 
 #include "compiledict.h"
-#include "dictionary.h"
-#include "readclause.h"
-
-#include "error.h"
-#include "speech.h"
-#include "phoneme.h"
-#include "voice.h"
-#include "synthesize.h"
-#include "translate.h"
+#include "dictionary.h"           // for EncodePhonemes, strncpy0, HashDicti...
+#include "error.h"                // for create_file_error_context
+#include "mnemonics.h"               // for LookupMnemName, MNEM_TAB
+#include "phoneme.h"              // for PHONEME_TAB_LIST, phonSWITCH, phone...
+#include "readclause.h"           // for towlower2
+#include "speech.h"		// for path_home
+#include "synthesize.h"           // for Write4Bytes
+#include "translate.h"            // for isspace2, IsDigit09, utf8_in, utf8_out
 
 static FILE *f_log = NULL;
 
@@ -922,7 +921,7 @@ static void copy_rule_string(char *string, int *state_out)
 					mr = mnem_rules;
 					while (mr->mnem != NULL) {
 						len = strlen(mr->mnem);
-						if (memcmp(p, mr->mnem, len) == 0) {
+						if (strncmp(p, mr->mnem, len) == 0) {
 							value = mr->value;
 							p += len;
 							break;
@@ -1559,7 +1558,9 @@ ESPEAK_NG_API espeak_ng_STATUS espeak_ng_CompileDictionary(const char *dsource,
 		fclose(f_in);
 		return create_file_error_context(context, error, fname_out);
 	}
-	sprintf(fname_temp, "%s%ctemp", path_home, PATHSEP);
+	/* Use dictionary-specific temp names to allow parallel compilation
+	 * of multiple ductionaries. */
+	sprintf(fname_temp, "%s%c%stemp", path_home, PATHSEP, dict_name);
 
 	value = N_HASH_DICT;
 	Write4Bytes(f_out, value);
