@@ -28,11 +28,11 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
-#include "speech.h"
-#include "phoneme.h"
-#include "synthesize.h"
-#include "voice.h"
-#include "translate.h"
+#include "intonation.h"
+#include "phoneme.h"     // for PHONEME_TAB, PhonemeCode2, phonPAUSE, phPAUSE
+#include "synthdata.h"   // for PhonemeCode
+#include "synthesize.h"  // for PHONEME_LIST, TUNE, phoneme_list, phoneme_tab
+#include "translate.h"   // for Translator, LANGUAGE_OPTIONS, L, OPTION_EMPH...
 
 /* Note this module is mostly old code that needs to be rewritten to
    provide a more flexible intonation system.
@@ -287,18 +287,6 @@ static TONE_NUCLEUS tone_nucleus_table[N_TONE_NUCLEUS_TABLE] = {
 	{ PITCHfall,   70, 18, PITCHfall,   70, 24, NULL, 32, 20, 0 },      // 12 test
 };
 
-// index by 0=. 1=, 2=?, 3=! 4=none, 5=emphasized
-unsigned char punctuation_to_tone[INTONATION_TYPES][PUNCT_INTONATIONS] = {
-	{  0,  1,  2,  3, 0, 4 },
-	{  0,  1,  2,  3, 0, 4 },
-	{  5,  6,  2,  3, 0, 4 },
-	{  5,  7,  1,  3, 0, 4 },
-	{  8,  9, 10,  3, 0, 0 },
-	{  8,  8, 10,  3, 0, 0 },
-	{ 11, 11, 11, 11, 0, 0 }, // 6 test
-	{ 12, 12, 12, 12, 0, 0 }
-};
-
 int n_tunes = 0;
 TUNE *tunes = NULL;
 
@@ -429,7 +417,7 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 	int increment = 0;
 	int n_steps = 0;
 	int stage; // onset, head, last
-	int initial;
+	bool initial;
 	int overflow_ix = 0;
 	int pitch_range;
 	int pitch_range_abs;
@@ -437,14 +425,14 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 	int n_unstressed = 0;
 	int unstressed_ix = 0;
 	int unstressed_inc;
-	int used_onset = 0;
+	bool used_onset = false;
 	int head_final = end_ix;
 	int secondary = 2;
 
-	pitch_range = (tune->head_end - tune->head_start) << 8;
+	pitch_range = (tune->head_end - tune->head_start) * 256;
 	pitch_range_abs = abs(pitch_range);
-	drops = drops_0; // this should be controled by tune->head_drops
-	initial = 1;
+	drops = drops_0; // this should be controlled by tune->head_drops
+	initial = true;
 
 	stage = 0;
 	if (tune->onset == 255)
@@ -468,17 +456,17 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 			// a primary stress
 
 			if ((initial) || (stress == 5)) {
-				initial = 0;
+				initial = false;
 				overflow_ix = 0;
 
 				if (tune->onset == 255) {
 					n_steps = count_increments(syllable_tab, syl_ix, head_final, 4);
-					pitch = tune->head_start << 8;
+					pitch = tune->head_start * 256;
 				} else {
 					// a pitch has been specified for the onset syllable, don't include it in the pitch incrementing
 					n_steps = count_increments(syllable_tab, syl_ix+1, head_final, 4);
-					pitch = tune->onset << 8;
-					used_onset = 1;
+					pitch = tune->onset * 256;
+					used_onset = true;
 				}
 
 				if (n_steps > tune->head_max_steps)
@@ -490,18 +478,18 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 					increment = 0;
 			} else if (syl_ix == head_final) {
 				// a pitch has been specified for the last primary stress before the nucleus
-				pitch = tune->head_last << 8;
+				pitch = tune->head_last * 256;
 				stage = 2;
 			} else {
 				if (used_onset) {
 					stage = 1;
-					used_onset = 0;
-					pitch = tune->head_start << 8;
+					used_onset = false;
+					pitch = tune->head_start * 256;
 					n_steps++;
 				} else if (n_steps > 0)
 					pitch += increment;
 				else {
-					pitch = (tune->head_end << 8) + (pitch_range_abs * tune->head_extend[overflow_ix++])/64;
+					pitch = (tune->head_end * 256) + (pitch_range_abs * tune->head_extend[overflow_ix++])/64;
 					if (overflow_ix >= tune->n_head_extend)
 						overflow_ix = 0;
 				}
@@ -515,18 +503,18 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 			unstressed_ix = 0;
 			syl->stress = PRIMARY_STRESSED;
 			syl->env = tune->stressed_env;
-			set_pitch(syl, (pitch >> 8), tune->stressed_drop);
+			set_pitch(syl, (pitch / 256), tune->stressed_drop);
 		} else if (stress >= secondary) {
 			n_unstressed = CountUnstressed(syllable_tab, syl_ix+1, end_ix, secondary);
 			unstressed_ix = 0;
-			set_pitch(syl, (pitch >> 8), drops[stress]);
+			set_pitch(syl, (pitch / 256), drops[stress]);
 		} else {
 			if (n_unstressed > 1)
 				unstressed_inc = (tune->unstr_end[stage] - tune->unstr_start[stage]) / (n_unstressed - 1);
 			else
 				unstressed_inc = 0;
 
-			set_pitch(syl, (pitch >> 8) + tune->unstr_start[stage] + (unstressed_inc * unstressed_ix), drops[stress]);
+			set_pitch(syl, (pitch / 256) + tune->unstr_start[stage] + (unstressed_inc * unstressed_ix), drops[stress]);
 			unstressed_ix++;
 		}
 
@@ -538,14 +526,14 @@ static int SetHeadIntonation(SYLLABLE *syllable_tab, TUNE *tune, int syl_ix, int
 /* Calculate pitches until next RESET or tonic syllable, or end.
     Increment pitch if stress is >= min_stress.
     Used for tonic segment */
-static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_HEAD *th, TONE_NUCLEUS *tn, int min_stress, int continuing)
+static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_HEAD *th, TONE_NUCLEUS *tn, int min_stress, bool continuing)
 {
 	int stress;
 	int pitch = 0;
 	int increment = 0;
 	int n_primary = 0;
 	int n_steps = 0;
-	int initial;
+	bool initial;
 	int overflow = 0;
 	int n_overflow;
 	int pitch_range;
@@ -557,11 +545,11 @@ static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_H
 	static signed char continue_tab[5] = { -26, 32, 20, 8, 0 };
 
 	drops = th->body_drops;
-	pitch_range = (th->body_end - th->body_start) << 8;
+	pitch_range = (th->body_end - th->body_start) * 256;
 	pitch_range_abs = abs(pitch_range);
 
 	if (continuing) {
-		initial = 0;
+		initial = false;
 		overflow = 0;
 		n_overflow = 5;
 		overflow_tab = continue_tab;
@@ -569,7 +557,7 @@ static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_H
 	} else {
 		n_overflow = th->n_overflow;
 		overflow_tab = th->overflow;
-		initial = 1;
+		initial = true;
 	}
 
 	while (ix < end_ix) {
@@ -580,7 +568,7 @@ static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_H
 			// a primary stress
 
 			if ((initial) || (stress == 5)) {
-				initial = 0;
+				initial = false;
 				overflow = 0;
 				n_steps = n_primary = count_increments(syllable_tab, ix, end_ix, min_stress);
 
@@ -592,12 +580,12 @@ static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_H
 				else
 					increment = 0;
 
-				pitch = th->body_start << 8;
+				pitch = th->body_start * 256;
 			} else {
 				if (n_steps > 0)
 					pitch += increment;
 				else {
-					pitch = (th->body_end << 8) + (pitch_range_abs * overflow_tab[overflow++])/64;
+					pitch = (th->body_end * 256) + (pitch_range_abs * overflow_tab[overflow++])/64;
 					if (overflow >= n_overflow) {
 						overflow = 0;
 						overflow_tab = th->overflow;
@@ -609,20 +597,20 @@ static int calc_pitch_segment(SYLLABLE *syllable_tab, int ix, int end_ix, TONE_H
 
 			n_primary--;
 			if ((tn->backwards) && (n_primary < 2))
-				pitch = tn->backwards[n_primary] << 8;
+				pitch = tn->backwards[n_primary] * 256;
 		}
 
 		if (stress >= PRIMARY) {
 			syl->stress = PRIMARY_STRESSED;
-			set_pitch(syl, (pitch >> 8), drops[stress]);
+			set_pitch(syl, (pitch / 256), drops[stress]);
 		} else if (stress >= SECONDARY)
-			set_pitch(syl, (pitch >> 8), drops[stress]);
+			set_pitch(syl, (pitch / 256), drops[stress]);
 		else {
 			// unstressed, drop pitch if preceded by PRIMARY
 			if ((syllable_tab[ix-1].stress & 0x3f) >= SECONDARY)
-				set_pitch(syl, (pitch >> 8) - th->body_lower_u, drops[stress]);
+				set_pitch(syl, (pitch / 256) - th->body_lower_u, drops[stress]);
 			else
-				set_pitch(syl, (pitch >> 8), drops[stress]);
+				set_pitch(syl, (pitch / 256), drops[stress]);
 		}
 
 		ix++;
@@ -643,7 +631,7 @@ static void SetPitchGradient(SYLLABLE *syllable_tab, int start_ix, int end_ix, i
 	int drop;
 	SYLLABLE *syl;
 
-	increment = (end_pitch - start_pitch) << 8;
+	increment = (end_pitch - start_pitch) * 256;
 	n_increments = end_ix - start_ix;
 
 	if (n_increments <= 0)
@@ -652,17 +640,17 @@ static void SetPitchGradient(SYLLABLE *syllable_tab, int start_ix, int end_ix, i
 	if (n_increments > 1)
 		increment = increment / n_increments;
 
-	pitch = start_pitch << 8;
+	pitch = start_pitch * 256;
 
 	for (ix = start_ix; ix < end_ix; ix++) {
 		syl = &syllable_tab[ix];
 		stress = syl->stress;
 
 		if (increment > 0) {
-			set_pitch(syl, (pitch >> 8), -(increment >> 8));
+			set_pitch(syl, (pitch / 256), -(increment / 256));
 			pitch += increment;
 		} else {
-			drop = -(increment >> 8);
+			drop = -(increment / 256);
 			if (drop < min_drop[stress])
 				drop = min_drop[stress];
 
@@ -670,7 +658,7 @@ static void SetPitchGradient(SYLLABLE *syllable_tab, int start_ix, int end_ix, i
 
 			if (drop > 18)
 				drop = 18;
-			set_pitch(syl, (pitch >> 8), drop);
+			set_pitch(syl, (pitch / 256), drop);
 		}
 	}
 }
@@ -729,13 +717,13 @@ static int calc_pitches(SYLLABLE *syllable_tab, int control, int start, int end,
 	TONE_HEAD *th;
 	TONE_NUCLEUS *tn;
 	int drop;
-	int continuing = 0;
+	bool continuing = false;
 
 	if (control == 0)
 		return calc_pitches2(syllable_tab, start, end, tune_number);
 
 	if (start > 0)
-		continuing = 1;
+		continuing = true;
 
 	th = &tone_head_table[tune_number];
 	tn = &tone_nucleus_table[tune_number];
@@ -789,14 +777,14 @@ static void CalcPitches_Tone(Translator *tr)
 	int final_stressed = 0;
 
 	int tone_ph;
-	int pause;
-	int tone_promoted;
+	bool pause;
+	bool tone_promoted;
 	PHONEME_TAB *tph;
 	PHONEME_TAB *prev_tph; // forget across word boundary
 	PHONEME_TAB *prevw_tph; // remember across word boundary
 	PHONEME_LIST *prev_p;
 
-	int pitch_adjust = 0;    // pitch gradient through the clause - inital value
+	int pitch_adjust = 0;    // pitch gradient through the clause - initial value
 	int pitch_decrement = 0; // decrease by this for each stressed syllable
 	int pitch_low = 0;       // until it drops to this
 	int pitch_high = 0;      // then reset to this
@@ -825,8 +813,8 @@ static void CalcPitches_Tone(Translator *tr)
 			p->tone_ph = PhonemeCode('7'); // change default tone (tone 1) to falling tone at end of clause
 	}
 
-	pause = 1;
-	tone_promoted = 0;
+	pause = true;
+	tone_promoted = false;
 
 	prev_p = p = &phoneme_list[0];
 	prev_tph = prevw_tph = phoneme_tab[phonPAUSE];
@@ -834,7 +822,7 @@ static void CalcPitches_Tone(Translator *tr)
 	// perform tone sandhi
 	for (ix = 0; ix < n_phoneme_list; ix++, p++) {
 		if ((p->type == phPAUSE) && (p->ph->std_length > 50)) {
-			pause = 1; // there is a pause since the previous vowel
+			pause = true; // there is a pause since the previous vowel
 			prevw_tph = phoneme_tab[phonPAUSE]; // forget previous tone
 		}
 
@@ -844,20 +832,33 @@ static void CalcPitches_Tone(Translator *tr)
 		if (p->synthflags & SFLAG_SYLLABLE) {
 			tone_ph = p->tone_ph;
 			tph = phoneme_tab[tone_ph];
-
+			
+			/* Hakka
+			ref.:https://en.wikipedia.org/wiki/Sixian_dialect#Tone_sandhi */
+			if (tr->translator_name == L3('h','a','k')){
+				if (prev_tph->mnemonic == 0x31){ // [previous one is 1st tone]
+				  // [this one is 1st, 4th, or 6th tone]
+				  if (tph->mnemonic == 0x31 || tph->mnemonic == 0x34 ||
+					  tph->mnemonic == 0x36){
+					/* trigger the tone sandhi of the prev. syllable
+					   from 1st tone ->2nd tone */
+					prev_p->tone_ph = PhonemeCode('2'); 
+				  }
+				}
+			  }
 			// Mandarin
-			if (tr->translator_name == L('z', 'h')) {
+			if (tr->translator_name == L('z', 'h') || tr->translator_name == L3('c', 'm', 'n')) {
 				if (tone_ph == 0) {
 					if (pause || tone_promoted) {
 						tone_ph = PhonemeCode2('5', '5'); // no previous vowel, use tone 1
-						tone_promoted = 1;
+						tone_promoted = true;
 					} else
 						tone_ph = PhonemeCode2('1', '1'); // default tone 5
 
 					p->tone_ph = tone_ph;
 					tph = phoneme_tab[tone_ph];
 				} else
-					tone_promoted = 0;
+					tone_promoted = false;
 
 				if (ix == final_stressed) {
 					if ((tph->mnemonic == 0x3535 ) || (tph->mnemonic == 0x3135)) {
@@ -891,7 +892,7 @@ static void CalcPitches_Tone(Translator *tr)
 
 			prev_p = p;
 			prevw_tph = prev_tph = tph;
-			pause = 0;
+			pause = false;
 		}
 	}
 
