@@ -36,17 +36,17 @@
 #include <espeak-ng/encoding.h>
 #include <ucd/ucd.h>
 
-#include "dictionary.h"
 #include "readclause.h"
-#include "synthdata.h"
-
-#include "error.h"
-#include "speech.h"
-#include "phoneme.h"
-#include "voice.h"
-#include "synthesize.h"
-#include "translate.h"
-#include "ssml.h"
+#include "config.h"               // for HAVE_MKSTEMP
+#include "dictionary.h"           // for LookupDictList, DecodePhonemes, Set...
+#include "error.h"                // for create_file_error_context
+#include "phoneme.h"              // for phonSWITCH
+#include "soundicon.h"               // for LookupSoundIcon
+#include "speech.h"               // for GetFileLength, LookupMnem, PATHSEP
+#include "ssml.h"                 // for SSML_STACK, ProcessSsmlTag, N_PARAM...
+#include "synthdata.h"            // for SelectPhonemeTable
+#include "translate.h"            // for Translator, utf8_out, CLAUSE_OPTION...
+#include "voice.h"                // for voice, voice_t, current_voice_selected
 
 #define N_XML_BUF   500
 
@@ -59,7 +59,6 @@ char *namedata = NULL;
 static int ungot_char2 = 0;
 espeak_ng_TEXT_DECODER *p_decoder = NULL;
 static int ungot_char;
-static const char *ungot_word = NULL;
 
 static bool ignore_text = false; // set during <sub> ... </sub>  to ignore text which has been replaced by an alias
 static bool audio_text = false; // set during <audio> ... </audio>
@@ -67,7 +66,6 @@ static bool clear_skipping_text = false; // next clause should clear the skippin
 int count_characters = 0;
 static int sayas_mode;
 static int sayas_start;
-static int ssml_ignore_l_angle = 0;
 
 #define N_SSML_STACK  20
 static int n_ssml_stack;
@@ -255,7 +253,7 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 
 	if ((only == 0) && ((phonemes[0] == 0) || (phonemes[0] == phonSWITCH)) && (tr->translator_name != L('e', 'n'))) {
 		// not found, try English
-		SetTranslator2("en");
+		SetTranslator2(ESPEAKNG_DEFAULT_VOICE);
 		string = &single_letter[1];
 		single_letter[1] = '_';
 		if (LookupDictList(translator2, &string, phonemes, flags, 0, NULL) == 0) {
@@ -263,7 +261,7 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 			LookupDictList(translator2, &string, phonemes, flags, 0, NULL);
 		}
 		if (phonemes[0])
-			lang_name = "en";
+			lang_name = ESPEAKNG_DEFAULT_VOICE;
 		else
 			SelectPhonemeTable(voice->phoneme_tab_ix); // revert to original phoneme table
 	}
@@ -272,7 +270,7 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 		if (lang_name) {
 			SetWordStress(translator2, phonemes, flags, -1, 0);
 			DecodePhonemes(phonemes, phonemes2);
-			sprintf(buf, "[\002_^_%s %s _^_%s]]", "en", phonemes2, WordToString2(tr->translator_name));
+			sprintf(buf, "[\002_^_%s %s _^_%s]]", ESPEAKNG_DEFAULT_VOICE, phonemes2, WordToString2(tr->translator_name));
 			SelectPhonemeTable(voice->phoneme_tab_ix); // revert to original phoneme table
 		} else {
 			SetWordStress(tr, phonemes, flags, -1, 0);
@@ -299,143 +297,6 @@ int Read4Bytes(FILE *f)
 	return acc;
 }
 
-static espeak_ng_STATUS LoadSoundFile(const char *fname, int index, espeak_ng_ERROR_CONTEXT *context)
-{
-	FILE *f;
-	char *p;
-	int *ip;
-	int length;
-	char fname_temp[100];
-	char fname2[sizeof(path_home)+13+40];
-
-	if (fname == NULL) {
-		// filename is already in the table
-		fname = soundicon_tab[index].filename;
-	}
-
-	if (fname == NULL)
-		return EINVAL;
-
-	if (fname[0] != '/') {
-		// a relative path, look in espeak-ng-data/soundicons
-		sprintf(fname2, "%s%csoundicons%c%s", path_home, PATHSEP, PATHSEP, fname);
-		fname = fname2;
-	}
-
-	f = NULL;
-	if ((f = fopen(fname, "rb")) != NULL) {
-		int ix;
-		int fd_temp;
-		int header[3];
-		char command[sizeof(fname2)+sizeof(fname2)+40];
-
-		if (fseek(f, 20, SEEK_SET) == -1) {
-			int error = errno;
-			fclose(f);
-			return create_file_error_context(context, error, fname);
-		}
-
-		for (ix = 0; ix < 3; ix++)
-			header[ix] = Read4Bytes(f);
-
-		// if the sound file is not mono, 16 bit signed, at the correct sample rate, then convert it
-		if ((header[0] != 0x10001) || (header[1] != samplerate) || (header[2] != samplerate*2)) {
-			fclose(f);
-			f = NULL;
-
-#ifdef HAVE_MKSTEMP
-			strcpy(fname_temp, "/tmp/espeakXXXXXX");
-			if ((fd_temp = mkstemp(fname_temp)) >= 0)
-				close(fd_temp);
-#else
-			strcpy(fname_temp, tmpnam(NULL));
-#endif
-
-			sprintf(command, "sox \"%s\" -r %d -c1 -t wav %s\n", fname, samplerate, fname_temp);
-			if (system(command) == 0)
-				fname = fname_temp;
-		}
-	}
-
-	if (f == NULL) {
-		f = fopen(fname, "rb");
-		if (f == NULL)
-			return create_file_error_context(context, errno, fname);
-	}
-
-	length = GetFileLength(fname);
-	if (length < 0) { // length == -errno
-		fclose(f);
-		return create_file_error_context(context, -length, fname);
-	}
-	if (fseek(f, 0, SEEK_SET) == -1) {
-		int error = errno;
-		fclose(f);
-		return create_file_error_context(context, error, fname);
-	}
-	if ((p = (char *)realloc(soundicon_tab[index].data, length)) == NULL) {
-		fclose(f);
-		return ENOMEM;
-	}
-	if (fread(p, 1, length, f) != length) {
-		int error = errno;
-		fclose(f);
-		remove(fname_temp);
-		free(p);
-		return create_file_error_context(context, error, fname);
-	}
-	fclose(f);
-	remove(fname_temp);
-
-	ip = (int *)(&p[40]);
-	soundicon_tab[index].length = (*ip) / 2; // length in samples
-	soundicon_tab[index].data = p;
-	return ENS_OK;
-}
-
-static int LookupSoundicon(int c)
-{
-	// Find the sound icon number for a punctuation chatacter
-	int ix;
-
-	for (ix = N_SOUNDICON_SLOTS; ix < n_soundicon_tab; ix++) {
-		if (soundicon_tab[ix].name == c) {
-			if (soundicon_tab[ix].length == 0) {
-				if (LoadSoundFile(NULL, ix, NULL) != ENS_OK)
-					return -1; // sound file is not available
-			}
-			return ix;
-		}
-	}
-	return -1;
-}
-
-int LoadSoundFile2(const char *fname)
-{
-	// Load a sound file into one of the reserved slots in the sound icon table
-	// (if it'snot already loaded)
-
-	int ix;
-	static int slot = -1;
-
-	for (ix = 0; ix < n_soundicon_tab; ix++) {
-		if (((soundicon_tab[ix].filename != NULL) && strcmp(fname, soundicon_tab[ix].filename) == 0))
-			return ix; // already loaded
-	}
-
-	// load the file into the next slot
-	slot++;
-	if (slot >= N_SOUNDICON_SLOTS)
-		slot = 0;
-
-	if (LoadSoundFile(fname, slot, NULL) != ENS_OK)
-		return -1;
-
-	soundicon_tab[slot].filename = (char *)realloc(soundicon_tab[ix].filename, strlen(fname)+1);
-	strcpy(soundicon_tab[slot].filename, fname);
-	return slot;
-}
-
 static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output, int *bufix, int end_clause)
 {
 	// announce punctuation names
@@ -504,10 +365,6 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 		} else {
 			// end the clause now and pick up the punctuation next time
 			UngetC(c2);
-			if (option_ssml) {
-				if ((c1 == '<') || (c1 == '&'))
-					ssml_ignore_l_angle = c1; // this was &lt; which was converted to <, don't pick it up again as <
-			}
 			ungot_char2 = c1;
 			buf[0] = ' ';
 			buf[1] = 0;
@@ -603,16 +460,6 @@ static void RemoveChar(char *p)
 	memset(p, ' ', utf8_in(&c, p));
 }
 
-static MNEM_TAB xml_char_mnemonics[] = {
-	{ "gt",   '>' },
-	{ "lt",   0xe000 + '<' },   // private usage area, to avoid confusion with XML tag
-	{ "amp",  '&' },
-	{ "quot", '"' },
-	{ "nbsp", ' ' },
-	{ "apos", '\'' },
-	{ NULL,   -1 }
-};
-
 int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change)
 {
 	/* Find the end of the current clause.
@@ -640,9 +487,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 	int phoneme_mode = 0;
 	int n_xml_buf;
 	int terminator;
-	int found;
 	bool any_alnum = false;
-	bool self_closing;
 	int punct_data = 0;
 	bool is_end_clause;
 	int announced_punctuation = 0;
@@ -667,16 +512,13 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 	*tone_type = 0;
 	*voice_change = 0;
 
-	if (ungot_word != NULL) {
-		strcpy(buf, ungot_word);
-		ix += strlen(ungot_word);
-		ungot_word = NULL;
-	}
-
-	if (ungot_char2 != 0)
+	if (ungot_char2 != 0) {
 		c2 = ungot_char2;
-	else
+	} else if (Eof()) {
+		c2 = 0;
+	} else {
 		c2 = GetC();
+	}
 
 	while (!Eof() || (ungot_char != 0) || (ungot_char2 != 0) || (ungot_string_ix >= 0)) {
 		if (!iswalnum(c1)) {
@@ -705,18 +547,18 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 		if ((ungot_string_ix == 0) && (ungot_char2 == 0))
 			c1 = ungot_string[ungot_string_ix++];
-		if (ungot_string_ix >= 0)
+		if (ungot_string_ix >= 0) {
 			c2 = ungot_string[ungot_string_ix++];
-		else {
+		} else if (Eof()) {
+			c2 = ' ';
+		} else {
 			c2 = GetC();
-
-			if (Eof())
-				c2 = ' ';
 		}
+
 		ungot_char2 = 0;
 
 		if ((option_ssml) && (phoneme_mode == 0)) {
-			if ((ssml_ignore_l_angle != '&') && (c1 == '&') && ((c2 == '#') || ((c2 >= 'a') && (c2 <= 'z')))) {
+			if ((c1 == '&') && ((c2 == '#') || ((c2 >= 'a') && (c2 <= 'z')))) {
 				n_xml_buf = 0;
 				c1 = c2;
 				while (!Eof() && (iswalnum(c1) || (c1 == '#')) && (n_xml_buf < N_XML_BUF2)) {
@@ -724,25 +566,17 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					c1 = GetC();
 				}
 				xml_buf2[n_xml_buf] = 0;
-				c2 = GetC();
+				if (Eof()) {
+					c2 = '\0';
+				} else {
+					c2 = GetC();
+				}
 				sprintf(ungot_string, "%s%c%c", &xml_buf2[0], c1, c2);
 
+				int found = -1;
 				if (c1 == ';') {
-					if (xml_buf2[0] == '#') {
-						// character code number
-						if (xml_buf2[1] == 'x')
-							found = sscanf(&xml_buf2[2], "%x", (unsigned int *)(&c1));
-						else
-							found = sscanf(&xml_buf2[1], "%d", &c1);
-					} else {
-						if ((found = LookupMnem(xml_char_mnemonics, xml_buf2)) != -1) {
-							c1 = found;
-							if (c2 == 0)
-								c2 = ' ';
-						}
-					}
-				} else
-					found = -1;
+					found = ParseSsmlReference(xml_buf2, &c1, &c2);
+				}
 
 				if (found <= 0) {
 					ungot_string_ix = 0;
@@ -752,13 +586,8 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 				if ((c1 <= 0x20) && ((sayas_mode == SAYAS_SINGLE_CHARS) || (sayas_mode == SAYAS_KEY)))
 					c1 += 0xe000; // move into unicode private usage area
-			} else if ((c1 == '<') && (ssml_ignore_l_angle != '<')) {
-				if ((c2 == '!') || (c2 == '?')) {
-					// a comment, ignore until closing '<'  (or <?xml tag )
-					while (!Eof() && (c1 != '>'))
-						c1 = GetC();
-					c2 = ' ';
-				} else if ((c2 == '/') || iswalpha(c2)) {
+			} else if (c1 == '<') {
+				if ((c2 == '/') || iswalpha(c2) || c2 == '!' || c2 == '?') {
 					// check for space in the output buffer for embedded commands produced by the SSML tag
 					if (ix > (n_buf - 20)) {
 						// Perhaps not enough room, end the clause before the SSML tag
@@ -779,14 +608,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					xml_buf[n_xml_buf] = 0;
 					c2 = ' ';
 
-					self_closing = false;
-					if (xml_buf[n_xml_buf-1] == '/') {
-						// a self-closing tag
-						xml_buf[n_xml_buf-1] = ' ';
-						self_closing = true;
-					}
-
-					terminator = ProcessSsmlTag(xml_buf, buf, &ix, n_buf, self_closing, xmlbase, &audio_text, current_voice_id, &base_voice, base_voice_variant_name, &ignore_text, &clear_skipping_text, &sayas_mode, &sayas_start, ssml_stack, &n_ssml_stack, &n_param_stack, (int *)speech_parameters);
+					terminator = ProcessSsmlTag(xml_buf, buf, &ix, n_buf, xmlbase, &audio_text, current_voice_id, &base_voice, base_voice_variant_name, &ignore_text, &clear_skipping_text, &sayas_mode, &sayas_start, ssml_stack, &n_ssml_stack, &n_param_stack, (int *)speech_parameters);
 
 					if (terminator != 0) {
 						buf[ix] = ' ';
@@ -797,12 +619,13 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 						return terminator;
 					}
 					c1 = ' ';
-					c2 = GetC();
+					if (!Eof()) {
+						c2 = GetC();
+					}
 					continue;
 				}
 			}
 		}
-		ssml_ignore_l_angle = 0;
 
 		if (ignore_text)
 			continue;
@@ -820,8 +643,8 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 			return terminator;
 		}
 
-		if ((c1 == CTRL_EMBEDDED) || (c1 == ctrl_embedded)) {
-			// an embedded command. If it's a voice change, end the clause
+		if (c1 == CTRL_EMBEDDED) {
+ 			// an embedded command. If it's a voice change, end the clause
 			if (c2 == 'V') {
 				buf[ix++] = 0; // end the clause at this point
 				while (!iswspace(c1 = GetC()) && !Eof() && (ix < (n_buf-1)))
@@ -879,26 +702,6 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 			if (c1 == 0xf0b)
 				c1 = ' '; // Tibet inter-syllabic mark, ?? replace by space ??
 
-			if (iswspace(c1)) {
-				char *p_word;
-
-				if (tr->translator_name == 0x6a626f) {
-					// language jbo : lojban
-					// treat "i" or ".i" as end-of-sentence
-					p_word = &buf[ix-1];
-					if (p_word[0] == 'i') {
-						if (p_word[-1] == '.')
-							p_word--;
-						if (p_word[-1] == ' ') {
-							ungot_word = "i ";
-							UngetC(c2);
-							p_word[0] = 0;
-							return CLAUSE_PERIOD;
-						}
-					}
-				}
-			}
-
 			if (c1 == 0xd4d) {
 				// Malayalam virama, check if next character is Zero-width-joiner
 				if (c2 == 0x200d)
@@ -908,11 +711,10 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 		if (iswupper(c1)) {
 			tr->clause_upper_count++;
+
 			if ((option_capitals == 2) && (sayas_mode == 0) && !iswupper(cprev)) {
-				char text_buf[40];
-				char text_buf2[30];
-				if (LookupSpecial(tr, "_cap", text_buf2) != NULL) {
-					sprintf(text_buf, "%s", text_buf2);
+				char text_buf[30];
+				if (LookupSpecial(tr, "_cap", text_buf) != NULL) {
 					j = strlen(text_buf);
 					if ((ix + j) < n_buf) {
 						strcpy(&buf[ix], text_buf);
@@ -1002,6 +804,16 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 			punct_data = 0;
 			if ((punct_data = clause_type_from_codepoint(c1)) != CLAUSE_NONE) {
+
+				// Handling of sequences of ? and ! like ??!?, !!??!, ?!! etc
+				// Use only first char as determinant
+				if(punct_data & (CLAUSE_QUESTION | CLAUSE_EXCLAMATION)) {
+					while(clause_type_from_codepoint(c2) & (CLAUSE_QUESTION | CLAUSE_EXCLAMATION)) {
+						c_next = GetC();
+						c2 = c_next;
+					}
+				}
+
 				if (punct_data & CLAUSE_PUNCTUATION_IN_WORD) {
 					// Armenian punctuation inside a word
 					stressed_word = true;
@@ -1009,7 +821,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					continue;
 				}
 
-				if ((iswspace(c2) || (punct_data & CLAUSE_OPTIONAL_SPACE_AFTER) || IsBracket(c2) || (c2 == '?') || Eof() || (c2 == ctrl_embedded))) { // don't check for '-' because it prevents recognizing ':-)'
+				if (iswspace(c2) || (punct_data & CLAUSE_OPTIONAL_SPACE_AFTER) || IsBracket(c2) || (c2 == '?') || Eof() || c2 == CTRL_EMBEDDED) { // don't check for '-' because it prevents recognizing ':-)'
 					// note: (c2='?') is for when a smart-quote has been replaced by '?'
 					is_end_clause = true;
 				}
@@ -1063,6 +875,12 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 						is_end_clause = false;
 					}
 
+					if (c1 == '.' && c_next == '\'' && text_decoder_peekc(p_decoder) == 's') {
+					 	// A special case to handle english acronym + genitive, eg. u.s.a.'s
+						// But avoid breaking clause handling if anything else follows the apostrophe.
+						is_end_clause = false;
+					}
+
 					if (c1 == '.') {
 						if ((tr->langopts.numbers & NUM_ORDINAL_DOT) &&
 						    (iswdigit(cprev) || (IsRomanU(cprev) && (IsRomanU(cprev2) || iswspace(cprev2))))) { // lang=hu
@@ -1071,11 +889,10 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 								is_end_clause = false; // Roman number followed by dot
 							else if (iswlower(c_next) || (c_next == '-')) // hyphen is needed for lang-hu (eg. 2.-kal)
 								is_end_clause = false; // only if followed by lower-case, (or if there is a XML tag)
-						} else if (c_next == '\'')
-							is_end_clause = false;    // eg. u.s.a.'s
-						if (iswlower(c_next)) {
+						} 
+						if (iswlower(c_next) && tr->langopts.lowercase_sentence == false) {
 							// next word has no capital letter, this dot is probably from an abbreviation
-							is_end_clause = 0;
+							is_end_clause = false;
 						}
 						if (any_alnum == false) {
 							// no letters or digits yet, so probably not a sentence terminator
