@@ -23,23 +23,25 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <wctype.h>
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
+
+#include "readclause.h"
+#include "setlengths.h"
+#include "synthdata.h"
+#include "wavegen.h"
 
-#include "speech.h"
 #include "phoneme.h"
-#include "synthesize.h"
 #include "voice.h"
+#include "synthesize.h"
 #include "translate.h"
 
-extern int GetAmplitude(void);
-extern void DoSonicSpeed(int value);
 extern int saved_parameters[];
 
 // convert from words-per-minute to internal speed factor
-// Use this to calibrate speed for wpm 80-350
+// Use this to calibrate speed for wpm 80-450 (espeakRATE_MINIMUM - espeakRATE_MAXIMUM)
 static unsigned char speed_lookup[] = {
 	255, 255, 255, 255, 255, //  80
 	253, 249, 245, 242, 238, //  85
@@ -147,8 +149,7 @@ void SetSpeed(int control)
 	int wpm_value;
 	double sonic;
 
-	speed.loud_consonants = 0;
-	speed.min_sample_len = 450;
+	speed.min_sample_len = espeakRATE_MAXIMUM;
 	speed.lenmod_factor = 110; // controls the effect of FRFLAG_LEN_MOD reduce length change
 	speed.lenmod2_factor = 100;
 	speed.min_pause = 5;
@@ -164,9 +165,9 @@ void SetSpeed(int control)
 
 	if (control & 2)
 		DoSonicSpeed(1 * 1024);
-	if ((wpm_value >= 450) || ((wpm_value > speed.fast_settings[0]) && (wpm > 350))) {
+	if ((wpm_value >= espeakRATE_MAXIMUM) || ((wpm_value > speed.fast_settings) && (wpm > 350))) {
 		wpm2 = wpm;
-		wpm = 175;
+		wpm = espeakRATE_NORMAL;
 
 		// set special eSpeak speed parameters for Sonic use
 		// The eSpeak output will be speeded up by at least x2
@@ -180,9 +181,9 @@ void SetSpeed(int control)
 			sonic = ((double)wpm2)/wpm;
 			DoSonicSpeed((int)(sonic * 1024));
 			speed.pause_factor = 85;
-			speed.clause_pause_factor = 80;
+			speed.clause_pause_factor = espeakRATE_MINIMUM;
 			speed.min_pause = 22;
-			speed.min_sample_len = 450*2;
+			speed.min_sample_len = espeakRATE_MAXIMUM*2;
 			speed.wav_factor = 211;
 			speed.lenmod_factor = 210;
 			speed.lenmod2_factor = 170;
@@ -190,16 +191,13 @@ void SetSpeed(int control)
 		return;
 	}
 
-	if (wpm > 450)
-		wpm = 450;
-
-	if (wpm > 360)
-		speed.loud_consonants = (wpm - 360) / 8;
+	if (wpm > espeakRATE_MAXIMUM)
+		wpm = espeakRATE_MAXIMUM;
 
 	wpm2 = wpm;
 	if (wpm > 359) wpm2 = 359;
-	if (wpm < 80) wpm2 = 80;
-	x = speed_lookup[wpm2-80];
+	if (wpm < espeakRATE_MINIMUM) wpm2 = espeakRATE_MINIMUM;
+	x = speed_lookup[wpm2-espeakRATE_MINIMUM];
 
 	if (wpm >= 380)
 		x = 7;
@@ -241,7 +239,7 @@ void SetSpeed(int control)
 			speed.wav_factor = wav_factor_350[wpm-350];
 
 		if (wpm >= 390) {
-			speed.min_sample_len = 450 - (wpm - 400)/2;
+			speed.min_sample_len = espeakRATE_MAXIMUM - (wpm - 400)/2;
 			if (wpm > 440)
 				speed.min_sample_len = 420 - (wpm - 440);
 		}
@@ -279,8 +277,7 @@ void SetSpeed(int control)
 	int wpm;
 	int wpm2;
 
-	speed.loud_consonants = 0;
-	speed.min_sample_len = 450;
+	speed.min_sample_len = espeakRATE_MAXIMUM;
 	speed.lenmod_factor = 110; // controls the effect of FRFLAG_LEN_MOD reduce length change
 	speed.lenmod2_factor = 100;
 
@@ -290,16 +287,13 @@ void SetSpeed(int control)
 
 	if (voice->speed_percent > 0)
 		wpm = (wpm * voice->speed_percent)/100;
-	if (wpm > 450)
-		wpm = 450;
-
-	if (wpm > 360)
-		speed.loud_consonants = (wpm - 360) / 8;
+	if (wpm > espeakRATE_MAXIMUM)
+		wpm = espeakRATE_MAXIMUM;
 
 	wpm2 = wpm;
 	if (wpm > 359) wpm2 = 359;
-	if (wpm < 80) wpm2 = 80;
-	x = speed_lookup[wpm2-80];
+	if (wpm < espeakRATE_MINIMUM) wpm2 = espeakRATE_MINIMUM;
+	x = speed_lookup[wpm2-espeakRATE_MINIMUM];
 
 	if (wpm >= 380)
 		x = 7;
@@ -341,7 +335,7 @@ void SetSpeed(int control)
 			speed.wav_factor = wav_factor_350[wpm-350];
 
 		if (wpm >= 390) {
-			speed.min_sample_len = 450 - (wpm - 400)/2;
+			speed.min_sample_len = espeakRATE_MAXIMUM - (wpm - 400)/2;
 			if (wpm > 440)
 				speed.min_sample_len = 420 - (wpm - 440);
 		}
@@ -375,6 +369,7 @@ espeak_ng_STATUS SetParameter(int parameter, int value, int relative)
 
 	int new_value = value;
 	int default_value;
+	extern const int param_defaults[N_SPEECH_PARAM];
 
 	if (relative) {
 		if (parameter < 5) {
@@ -453,8 +448,8 @@ void CalcLengths(Translator *tr)
 	int stress;
 	int type;
 	static int more_syllables = 0;
-	int pre_sonorant = 0;
-	int pre_voiced = 0;
+	bool pre_sonorant = false;
+	bool pre_voiced = false;
 	int last_pitch = 0;
 	int pitch_start;
 	int length_mod;
@@ -519,7 +514,7 @@ void CalcLengths(Translator *tr)
 					p->prepause = 15;
 			}
 
-			if (next->type == phPAUSE && prev->type == phNASAL && !(p->ph->phflags&phFORTIS))
+			if (next->type == phPAUSE && prev->type == phNASAL && !(p->ph->phflags&phVOICELESS))
 				p->prepause = 25;
 
 			if (prev->ph->phflags & phBRKAFTER)
@@ -538,7 +533,7 @@ void CalcLengths(Translator *tr)
 
 			if (type == phVFRICATIVE) {
 				if (next->type == phVOWEL)
-					pre_voiced = 1;
+					pre_voiced = true;
 				if ((prev->type == phVOWEL) || (prev->type == phLIQUID))
 					p->length = (255 + prev->length)/2;
 			}
@@ -549,7 +544,7 @@ void CalcLengths(Translator *tr)
 
 			if (next->type == phVOWEL || next->type == phLIQUID) {
 				if ((next->type == phVOWEL) || !next->newword)
-					pre_voiced = 1;
+					pre_voiced = true;
 
 				p->prepause = 40;
 
@@ -567,7 +562,7 @@ void CalcLengths(Translator *tr)
 					if (prev->type == phNASAL)
 						p->prepause = 12;
 
-					if (prev->type == phSTOP && !(prev->ph->phflags & phFORTIS))
+					if (prev->type == phSTOP && !(prev->ph->phflags & phVOICELESS))
 						p->prepause = 0;
 				}
 			}
@@ -589,7 +584,7 @@ void CalcLengths(Translator *tr)
 			}
 
 			if (next->type == phVOWEL)
-				pre_sonorant = 1;
+				pre_sonorant = true;
 			else {
 				p->pitch2 = last_pitch;
 
@@ -616,7 +611,7 @@ void CalcLengths(Translator *tr)
 				if (p->pitch2 < 16)
 					p->pitch1 = 0;
 				p->env = PITCHfall;
-				pre_voiced = 0;
+				pre_voiced = false;
 			}
 			break;
 		case phVOWEL:
@@ -655,7 +650,7 @@ void CalcLengths(Translator *tr)
 			if (p2->ph->code == phonPAUSE_CLAUSE)
 				end_of_clause = 2;
 
-			if ((p2->newword & 2) && (more_syllables == 0))
+			if ((p2->newword & PHLIST_END_OF_CLAUSE) && (more_syllables == 0))
 				end_of_clause = 2;
 
 			// calc length modifier
@@ -684,7 +679,7 @@ void CalcLengths(Translator *tr)
 			} else {
 				length_mod = tr->langopts.length_mods[next2type *10+ next->ph->length_mod];
 
-				if ((next->type == phNASAL) && (next2->type == phSTOP || next2->type == phVSTOP) && (next3->ph->phflags & phFORTIS))
+				if ((next->type == phNASAL) && (next2->type == phSTOP || next2->type == phVSTOP) && (next3->ph->phflags & phVOICELESS))
 					length_mod -= 15;
 			}
 
@@ -806,9 +801,75 @@ void CalcLengths(Translator *tr)
 			}
 
 			last_pitch = p->pitch1 + ((p->pitch2-p->pitch1)*envelope_data[p->env][127])/256;
-			pre_sonorant = 0;
-			pre_voiced = 0;
+			pre_sonorant = false;
+			pre_voiced = false;
 			break;
 		}
 	}
 }
+// Tables of the relative lengths of vowels, depending on the
+// type of the two phonemes that follow
+// indexes are the "length_mod" value for the following phonemes
+
+// use this table if vowel is not the last in the word
+static unsigned char length_mods_en[100] = {
+//	a    ,    t    s    n    d    z    r    N    <- next
+	100, 120, 100, 105, 100, 110, 110, 100,  95, 100, // a  <- next2
+	105, 120, 105, 110, 125, 130, 135, 115, 125, 100, // ,
+	105, 120,  75, 100,  75, 105, 120,  85,  75, 100, // t
+	105, 120,  85, 105,  95, 115, 120, 100,  95, 100, // s
+	110, 120,  95, 105, 100, 115, 120, 100, 100, 100, // n
+	105, 120, 100, 105,  95, 115, 120, 110,  95, 100, // d
+	105, 120, 100, 105, 105, 122, 125, 110, 105, 100, // z
+	105, 120, 100, 105, 105, 122, 125, 110, 105, 100, // r
+	105, 120,  95, 105, 100, 115, 120, 110, 100, 100, // N
+	100, 120, 100, 100, 100, 100, 100, 100, 100, 100
+};
+
+// as above, but for the last syllable in a word
+static unsigned char length_mods_en0[100] = {
+//	a    ,    t    s    n    d    z    r    N    <- next
+	100, 150, 100, 105, 110, 115, 110, 110, 110, 100, // a  <- next2
+	105, 150, 105, 110, 125, 135, 140, 115, 135, 100, // ,
+	105, 150,  90, 105,  90, 122, 135, 100,  90, 100, // t
+	105, 150, 100, 105, 100, 122, 135, 100, 100, 100, // s
+	105, 150, 100, 105, 105, 115, 135, 110, 105, 100, // n
+	105, 150, 100, 105, 105, 122, 130, 120, 125, 100, // d
+	105, 150, 100, 105, 110, 122, 125, 115, 110, 100, // z
+	105, 150, 100, 105, 105, 122, 135, 120, 105, 100, // r
+	105, 150, 100, 105, 105, 115, 135, 110, 105, 100, // N
+	100, 100, 100, 100, 100, 100, 100, 100, 100, 100
+};
+
+
+static unsigned char length_mods_equal[100] = {
+//	a    ,    t    s    n    d    z    r    N    <- next
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // a  <- next2
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // ,
+	110, 120, 100, 110, 100, 110, 110, 110, 100, 110, // t
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // s
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // n
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // d
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // z
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // r
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // N
+	110, 120, 100, 110, 110, 110, 110, 110, 110, 110
+};
+
+static unsigned char *length_mod_tabs[6] = {
+	length_mods_en,
+	length_mods_en,    // 1
+	length_mods_en0,   // 2
+	length_mods_equal, // 3
+	length_mods_equal, // 4
+	length_mods_equal  // 5
+};
+
+void SetLengthMods(Translator *tr, int value)
+{
+	int value2;
+
+	tr->langopts.length_mods0 = tr->langopts.length_mods = length_mod_tabs[value % 100];
+	if ((value2 = value / 100) != 0)
+		tr->langopts.length_mods0 = length_mod_tabs[value2];
+}
