@@ -22,6 +22,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <math.h>
+#include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,22 +32,24 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
-#include "speech.h"
-#include "phoneme.h"
 #include "synthesize.h"
-#include "voice.h"
-#include "translate.h"
+#include "dictionary.h"           // for WritePhMnemonic, GetTranslatedPhone...
+#include "intonation.h"           // for CalcPitches
+#include "mbrola.h"               // for MbrolaGenerate, mbrola_name
+#include "phoneme.h"              // for PHONEME_TAB, phVOWEL, phLIQUID, phN...
+#include "setlengths.h"           // for CalcLengths
+#include "soundicon.h"               // for soundicon_tab, n_soundicon
+#include "synthdata.h"            // for InterpretPhoneme, GetEnvelope, Inte...
+#include "translate.h"            // for translator, LANGUAGE_OPTIONS, Trans...
+#include "voice.h"                // for voice_t, voice, LoadVoiceVariant
+#include "wavegen.h"              // for WcmdqInc, WcmdqFree, WcmdqStop
 
-extern FILE *f_log;
 static void SmoothSpect(void);
 
 // list of phonemes in a clause
 int n_phoneme_list = 0;
 PHONEME_LIST phoneme_list[N_PHONEME_LIST+1];
 
-int mbrola_delay;
-char mbrola_name[20];
-
 SPEED_FACTORS speed;
 
 static int last_pitch_cmd;
@@ -64,17 +67,10 @@ static int syllable_centre;
 
 static voice_t *new_voice = NULL;
 
-int n_soundicon_tab = N_SOUNDICON_SLOTS;
-SOUND_ICON soundicon_tab[N_SOUNDICON_TAB];
-
 #define RMS_GLOTTAL1 35   // vowel before glottal stop
 #define RMS_START 28  // 28
-
 #define VOWEL_FRONT_LENGTH  50
 
-// a dummy phoneme_list entry which looks like a pause
-static PHONEME_LIST next_pause;
-
 const char *WordToString(unsigned int word)
 {
 	// Convert a phoneme mnemonic word into a string
@@ -93,10 +89,6 @@ void SynthesizeInit()
 	last_amp_cmd = 0;
 	last_frame = NULL;
 	syllable_centre = -1;
-
-	// initialise next_pause, a dummy phoneme_list entry
-	next_pause.type = phPAUSE;
-	next_pause.newword = 0;
 }
 
 static void EndAmplitude(void)
@@ -110,7 +102,7 @@ static void EndAmplitude(void)
 
 static void EndPitch(int voice_break)
 {
-	// posssible end of pitch envelope, fill in the length
+	// possible end of pitch envelope, fill in the length
 	if ((pitch_length > 0) && (last_pitch_cmd >= 0)) {
 		if (wcmdq[last_pitch_cmd][1] == 0)
 			wcmdq[last_pitch_cmd][1] = pitch_length;
@@ -123,7 +115,6 @@ static void EndPitch(int voice_break)
 		syllable_end = wcmdq_tail;
 		SmoothSpect();
 		syllable_centre = -1;
-		memset(vowel_transition, 0, sizeof(vowel_transition));
 	}
 }
 
@@ -142,6 +133,14 @@ static void DoAmplitude(int amp, unsigned char *amp_env)
 	WcmdqInc();
 }
 
+static void DoPhonemeAlignment(char* pho, int type)
+{
+	wcmdq[wcmdq_tail][0] = WCMD_PHONEME_ALIGNMENT;
+	wcmdq[wcmdq_tail][1] = (intptr_t)pho;
+	wcmdq[wcmdq_tail][2] = type;
+	WcmdqInc();
+}
+
 static void DoPitch(unsigned char *env, int pitch1, int pitch2)
 {
 	intptr_t *q;
@@ -674,7 +673,7 @@ static void SmoothSpect(void)
 	int ix;
 	int len;
 	int pk;
-	int modified;
+	bool modified;
 	int allowed;
 	int diff;
 
@@ -710,7 +709,7 @@ static void SmoothSpect(void)
 				break; // doesn't follow on from previous frame
 
 			frame = frame2 = (frame_t *)q[2];
-			modified = 0;
+			modified = false;
 
 			if (frame->frflags & FRFLAG_BREAK)
 				break;
@@ -739,16 +738,16 @@ static void SmoothSpect(void)
 				allowed = (allowed * len)/256;
 
 				if (diff > allowed) {
-					if (modified == 0) {
+					if (modified == false) {
 						frame2 = CopyFrame(frame, 0);
-						modified = 1;
+						modified = true;
 					}
 					frame2->ffreq[pk] = frame1->ffreq[pk] + allowed;
 					q[2] = (intptr_t)frame2;
 				} else if (diff < -allowed) {
-					if (modified == 0) {
+					if (modified == false) {
 						frame2 = CopyFrame(frame, 0);
-						modified = 1;
+						modified = true;
 					}
 					frame2->ffreq[pk] = frame1->ffreq[pk] - allowed;
 					q[2] = (intptr_t)frame2;
@@ -784,7 +783,7 @@ static void SmoothSpect(void)
 			}
 
 			frame = frame2 = (frame_t *)q[3];
-			modified = 0;
+			modified = false;
 
 			if (frame1->frflags & FRFLAG_BREAK)
 				break;
@@ -806,16 +805,16 @@ static void SmoothSpect(void)
 				allowed = (allowed * len)/256;
 
 				if (diff > allowed) {
-					if (modified == 0) {
+					if (modified == false) {
 						frame2 = CopyFrame(frame, 0);
-						modified = 1;
+						modified = true;
 					}
 					frame2->ffreq[pk] = frame1->ffreq[pk] + allowed;
 					q[3] = (intptr_t)frame2;
 				} else if (diff < -allowed) {
-					if (modified == 0) {
+					if (modified == false) {
 						frame2 = CopyFrame(frame, 0);
-						modified = 1;
+						modified = true;
 					}
 					frame2->ffreq[pk] = frame1->ffreq[pk] - allowed;
 					q[3] = (intptr_t)frame2;
@@ -901,14 +900,7 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 	if (voice->klattv[0])
 		wcmd_spect = WCMD_KLATT;
 
-	wavefile_ix = fmt_params->wav_addr;
-
-	if (fmt_params->wav_amp == 0)
-		wavefile_amp = 32;
-	else
-		wavefile_amp = (fmt_params->wav_amp * 32)/100;
-
-	if (wavefile_ix == 0) {
+	if (fmt_params->wav_addr == 0) {
 		if (wave_flag) {
 			// cancel any wavefile that was playing previously
 			wcmd_spect = WCMD_SPECT2;
@@ -975,9 +967,15 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 		if ((fmt_params->wav_addr != 0) && ((frame1->frflags & FRFLAG_DEFER_WAV) == 0)) {
 			// there is a wave file to play along with this synthesis
 			seq_len_adjust = 0;
+	
+			int wavefile_amp;
+			if (fmt_params->wav_amp == 0)
+				wavefile_amp = 32;
+			else
+				wavefile_amp = (fmt_params->wav_amp * 32)/100;
+
 			DoSample2(fmt_params->wav_addr, which+0x100, 0, fmt_params->fmt_control, 0, wavefile_amp);
 			wave_flag = 1;
-			wavefile_ix = 0;
 			fmt_params->wav_addr = 0;
 		}
 
@@ -1127,7 +1125,9 @@ void DoEmbedded(int *embix, int sourceix)
 	} while ((word & 0x80) == 0);
 }
 
-int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
+extern espeak_ng_OUTPUT_HOOKS* output_hooks;
+
+int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 {
 	static int ix;
 	static int embedded_ix;
@@ -1136,17 +1136,17 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 	PHONEME_LIST *next;
 	PHONEME_LIST *next2;
 	PHONEME_LIST *p;
-	int released;
+	bool released;
 	int stress;
 	int modulation;
-	int pre_voiced;
+	bool  pre_voiced;
 	int free_min;
 	int value;
 	unsigned char *pitch_env = NULL;
 	unsigned char *amp_env;
 	PHONEME_TAB *ph;
 	int use_ipa = 0;
-	int done_phoneme_marker;
+	bool done_phoneme_marker;
 	int vowelstart_prev;
 	char phoneme_name[16];
 	static int sourceix = 0;
@@ -1164,7 +1164,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 	if (mbrola_name[0] != 0)
 		return MbrolaGenerate(phoneme_list, n_ph, resume);
 
-	if (resume == 0) {
+	if (resume == false) {
 		ix = 1;
 		embedded_ix = 0;
 		word_count = 0;
@@ -1176,13 +1176,20 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		syllable_end = wcmdq_tail;
 		syllable_centre = -1;
 		last_pitch_cmd = -1;
-		memset(vowel_transition, 0, sizeof(vowel_transition));
 		memset(&worddata, 0, sizeof(worddata));
 		DoPause(0, 0); // isolate from the previous clause
 	}
 
 	while ((ix < (*n_ph)) && (ix < N_PHONEME_LIST-2)) {
 		p = &phoneme_list[ix];
+		
+		if(output_hooks && output_hooks->outputPhoSymbol)
+		{
+			char buf[30];
+			int dummy=0;
+			WritePhMnemonic(buf, p->ph, p, 0, &dummy);
+			DoPhonemeAlignment(strdup(buf),p->type);
+		}
 
 		if (p->type == phPAUSE)
 			free_min = 10;
@@ -1209,10 +1216,10 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 
 			sourceix = (p->sourceix & 0x7ff) + clause_start_char;
 
-			if (p->newword & 4)
+			if (p->newword & PHLIST_START_OF_SENTENCE)
 				DoMarker(espeakEVENT_SENTENCE, sourceix, 0, count_sentences); // start of sentence
 
-			if (p->newword & 1)
+			if (p->newword & PHLIST_START_OF_WORD)
 				DoMarker(espeakEVENT_WORD, sourceix, p->sourceix >> 11, clause_start_word + word_count++); // NOTE, this count doesn't include multiple-word pronunciations in *_list. eg (of a)
 		}
 
@@ -1221,14 +1228,14 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		if ((p->prepause > 0) && !(p->ph->phflags & phPREVOICE))
 			DoPause(p->prepause, 1);
 
-		done_phoneme_marker = 0;
+		done_phoneme_marker = false;
 		if (option_phoneme_events && (p->ph->code != phonEND_WORD)) {
 			if ((p->type == phVOWEL) && (prev->type == phLIQUID || prev->type == phNASAL)) {
 				// For vowels following a liquid or nasal, do the phoneme event after the vowel-start
 			} else {
 				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
-				done_phoneme_marker = 1;
+				done_phoneme_marker = true;
 			}
 		}
 
@@ -1239,14 +1246,14 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			p->std_length = p->ph->std_length;
 			break;
 		case phSTOP:
-			released = 0;
+			released = false;
 			ph = p->ph;
 			if (next->type == phVOWEL)
-				released = 1;
+				released = true;
 			else if (!next->newword) {
-				if (next->type == phLIQUID) released = 1;
+				if (next->type == phLIQUID) released = true;
 			}
-			if (released == 0)
+			if (released == false)
 				p->synthflags |= SFLAG_NEXT_PAUSE;
 
 			if (ph->phflags & phPREVOICE) {
@@ -1280,15 +1287,15 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 			memset(&fmtp, 0, sizeof(fmtp));
 			fmtp.fmt_control = pd_DONTLENGTHEN;
 
-			pre_voiced = 0;
+			pre_voiced = false;
 			if (next->type == phVOWEL) {
 				DoAmplitude(p->amp, NULL);
 				DoPitch(envelope_data[p->env], p->pitch1, p->pitch2);
-				pre_voiced = 1;
+				pre_voiced = true;
 			} else if ((next->type == phLIQUID) && !next->newword) {
 				DoAmplitude(next->amp, NULL);
 				DoPitch(envelope_data[next->env], next->pitch1, next->pitch2);
-				pre_voiced = 1;
+				pre_voiced = true;
 			} else {
 				if (last_pitch_cmd < 0) {
 					DoAmplitude(next->amp, NULL);
@@ -1489,7 +1496,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 				DoSpect2(ph, 1, &fmtp, p, modulation);
 			}
 
-			if ((option_phoneme_events) && (done_phoneme_marker == 0)) {
+			if ((option_phoneme_events) && (done_phoneme_marker == false)) {
 				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
 			}
@@ -1550,7 +1557,7 @@ int SpeakNextClause(int control)
 	}
 
 	if (text_decoder_eof(p_decoder)) {
-		skipping_text = 0;
+		skipping_text = false;
 		return 0;
 	}
 
