@@ -32,17 +32,16 @@
 #include <espeak-ng/speak_lib.h>
 #include <espeak-ng/encoding.h>
 
-#include "dictionary.h"
-#include "numbers.h"
-#include "phonemelist.h"
-#include "readclause.h"
-#include "synthdata.h"
-
-#include "speech.h"
-#include "phoneme.h"
-#include "voice.h"
-#include "synthesize.h"
 #include "translate.h"
+#include "dictionary.h"           // for TranslateRules, LookupDictList, Cha...
+#include "numbers.h"              // for SetSpellingStress, TranslateLetter
+#include "phoneme.h"              // for phonSWITCH, PHONEME_TAB, phonPAUSE_...
+#include "phonemelist.h"          // for MakePhonemeList
+#include "readclause.h"           // for towlower2, Eof, ReadClause, is_str_...
+#include "synthdata.h"            // for SelectPhonemeTable, LookupPhonemeTable
+#include "synthesize.h"           // for PHONEME_LIST2, N_PHONEME_LIST, PHON...
+#include "ucd/ucd.h"              // for ucd_toupper
+#include "voice.h"                // for voice, voice_t
 
 Translator *translator = NULL; // the main translator
 Translator *translator2 = NULL; // secondary translator for certain words
@@ -89,7 +88,6 @@ int n_ph_list2;
 PHONEME_LIST2 ph_list2[N_PHONEME_LIST]; // first stage of text->phonemes
 
 wchar_t option_punctlist[N_PUNCTLIST] = { 0 };
-char ctrl_embedded = '\001'; // to allow an alternative CTRL for embedded commands
 
 // these are overridden by defaults set in the "speak" file
 int option_linelength = 0;
@@ -117,73 +115,6 @@ static const unsigned short brackets[] = {
 // other characters which break a word, but don't produce a pause
 static const unsigned short breaks[] = { '_', 0 };
 
-// Tables of the relative lengths of vowels, depending on the
-// type of the two phonemes that follow
-// indexes are the "length_mod" value for the following phonemes
-
-// use this table if vowel is not the last in the word
-static unsigned char length_mods_en[100] = {
-//	a    ,    t    s    n    d    z    r    N    <- next
-	100, 120, 100, 105, 100, 110, 110, 100,  95, 100, // a  <- next2
-	105, 120, 105, 110, 125, 130, 135, 115, 125, 100, // ,
-	105, 120,  75, 100,  75, 105, 120,  85,  75, 100, // t
-	105, 120,  85, 105,  95, 115, 120, 100,  95, 100, // s
-	110, 120,  95, 105, 100, 115, 120, 100, 100, 100, // n
-	105, 120, 100, 105,  95, 115, 120, 110,  95, 100, // d
-	105, 120, 100, 105, 105, 122, 125, 110, 105, 100, // z
-	105, 120, 100, 105, 105, 122, 125, 110, 105, 100, // r
-	105, 120,  95, 105, 100, 115, 120, 110, 100, 100, // N
-	100, 120, 100, 100, 100, 100, 100, 100, 100, 100
-};
-
-// as above, but for the last syllable in a word
-static unsigned char length_mods_en0[100] = {
-//	a    ,    t    s    n    d    z    r    N    <- next
-	100, 150, 100, 105, 110, 115, 110, 110, 110, 100, // a  <- next2
-	105, 150, 105, 110, 125, 135, 140, 115, 135, 100, // ,
-	105, 150,  90, 105,  90, 122, 135, 100,  90, 100, // t
-	105, 150, 100, 105, 100, 122, 135, 100, 100, 100, // s
-	105, 150, 100, 105, 105, 115, 135, 110, 105, 100, // n
-	105, 150, 100, 105, 105, 122, 130, 120, 125, 100, // d
-	105, 150, 100, 105, 110, 122, 125, 115, 110, 100, // z
-	105, 150, 100, 105, 105, 122, 135, 120, 105, 100, // r
-	105, 150, 100, 105, 105, 115, 135, 110, 105, 100, // N
-	100, 100, 100, 100, 100, 100, 100, 100, 100, 100
-};
-
-
-static unsigned char length_mods_equal[100] = {
-//	a    ,    t    s    n    d    z    r    N    <- next
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // a  <- next2
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // ,
-	110, 120, 100, 110, 100, 110, 110, 110, 100, 110, // t
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // s
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // n
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // d
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // z
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // r
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110, // N
-	110, 120, 100, 110, 110, 110, 110, 110, 110, 110
-};
-
-static unsigned char *length_mod_tabs[6] = {
-	length_mods_en,
-	length_mods_en,    // 1
-	length_mods_en0,   // 2
-	length_mods_equal, // 3
-	length_mods_equal, // 4
-	length_mods_equal  // 5
-};
-
-void SetLengthMods(Translator *tr, int value)
-{
-	int value2;
-
-	tr->langopts.length_mods0 = tr->langopts.length_mods = length_mod_tabs[value % 100];
-	if ((value2 = value / 100) != 0)
-		tr->langopts.length_mods0 = length_mod_tabs[value2];
-}
-
 int IsAlpha(unsigned int c)
 {
 	// Replacement for iswalph() which also checks for some in-word symbols
@@ -223,9 +154,6 @@ int IsAlpha(unsigned int c)
 	if ((c >= 0x300) && (c <= 0x36f))
 		return 1; // combining accents
 
-	if ((c >= 0x780) && (c <= 0x7b1))
-		return 1; // taani/divehi (maldives)
-
 	if ((c >= 0xf40) && (c <= 0xfbc))
 		return 1; // tibetan
 
@@ -447,6 +375,26 @@ char *strchr_w(const char *s, int c)
 	return strchr((char *)s, c); // (char *) is needed for Borland compiler
 }
 
+// append plural suffixes depending on preceding letter
+static void addPluralSuffixes(int flags, Translator *tr, char last_char, char *word_phonemes)
+{
+	char word_zz[4] = { ' ', 'z', 'z', 0 };
+	char word_iz[4] = { ' ', 'i', 'z', 0 };
+	char word_ss[4] = { ' ', 's', 's', 0 };
+	if (flags & FLAG_HAS_PLURAL) {
+		// s or 's suffix, append [s], [z] or [Iz] depending on previous letter
+		if (last_char == 'f')
+			TranslateRules(tr, &word_ss[1], word_phonemes, N_WORD_PHONEMES,
+			NULL, 0, NULL);
+		else if ((last_char == 0) || (strchr_w("hsx", last_char) == NULL))
+			TranslateRules(tr, &word_zz[1], word_phonemes, N_WORD_PHONEMES,
+			NULL, 0, NULL);
+		else
+			TranslateRules(tr, &word_iz[1], word_phonemes, N_WORD_PHONEMES,
+			NULL, 0, NULL);
+	}
+}
+
 static char *SpeakIndividualLetters(Translator *tr, char *word, char *phonemes, int spell_word)
 {
 	int posn = 0;
@@ -456,7 +404,7 @@ static char *SpeakIndividualLetters(Translator *tr, char *word, char *phonemes,
 	if (spell_word > 2)
 		capitals = 2; // speak 'capital'
 	if (spell_word > 1)
-		capitals |= 4; // speak charater code for unknown letters
+		capitals |= 4; // speak character code for unknown letters
 
 	while ((*word != ' ') && (*word != 0)) {
 		word += TranslateLetter(tr, word, phonemes, capitals | non_initial, current_alphabet);
@@ -493,7 +441,7 @@ static int CheckDottedAbbrev(char *word1)
 			if (word[nbytes+1] == '.') {
 				if (word[nbytes+2] == ' ')
 					ok = 1;
-				else if (word[nbytes+2] == '\'') {
+				else if (word[nbytes+2] == '\'' && word[nbytes+3] == 's') {
 					nbytes += 2; // delete the final dot (eg. u.s.a.'s)
 					ok = 2;
 				}
@@ -570,12 +518,6 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 	int add_suffix_phonemes = 0;
 	WORD_TAB wtab_null[8];
 
-	// translate these to get pronunciations of plural 's' suffix (different forms depending on
-	// the preceding letter
-	static char word_zz[4] = { 0, 'z', 'z', 0 };
-	static char word_iz[4] = { 0, 'i', 'z', 0 };
-	static char word_ss[4] = { 0, 's', 's', 0 };
-
 	if (wtab == NULL) {
 		memset(wtab_null, 0, sizeof(wtab_null));
 		wtab = wtab_null;
@@ -738,6 +680,8 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 		strcpy(word_phonemes, phonemes);
 		if (wflags & FLAG_TRANSLATOR2)
 			return 0;
+
+		addPluralSuffixes(wflags, tr, last_char, word_phonemes);
 		return dictionary_flags[0] & FLAG_SKIPWORDS; // for "b.c.d"
 	} else if (found == false) {
 		// word's pronunciation is not given in the dictionary list, although
@@ -768,7 +712,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 			if (unpron_phonemes[0] == phonSWITCH) {
 				// change to another language in order to translate this word
 				strcpy(word_phonemes, unpron_phonemes);
-				if (strcmp(&unpron_phonemes[1], "en") == 0)
+				if (strcmp(&unpron_phonemes[1], ESPEAKNG_DEFAULT_VOICE) == 0)
 					return FLAG_SPELLWORD; // _^_en must have been set in TranslateLetter(), not *_rules which uses only _^_
 				return 0;
 			}
@@ -1010,16 +954,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 		}
 	}
 
-	if (wflags & FLAG_HAS_PLURAL) {
-		// s or 's suffix, append [s], [z] or [Iz] depending on previous letter
-		if (last_char == 'f')
-			TranslateRules(tr, &word_ss[1], phonemes, N_WORD_PHONEMES, NULL, 0, NULL);
-		else if ((last_char == 0) || (strchr_w("hsx", last_char) == NULL))
-			TranslateRules(tr, &word_zz[1], phonemes, N_WORD_PHONEMES, NULL, 0, NULL);
-		else
-			TranslateRules(tr, &word_iz[1], phonemes, N_WORD_PHONEMES, NULL, 0, NULL);
-	}
-
+	addPluralSuffixes(wflags, tr, last_char, word_phonemes);
 	wflags |= emphasize_allcaps;
 
 	// determine stress pattern for this word
@@ -1154,6 +1089,7 @@ static int TranslateWord3(Translator *tr, char *word_start, WORD_TAB *wtab, char
 	return dictionary_flags[0];
 }
 
+
 int TranslateWord(Translator *tr, char *word_start, WORD_TAB *wtab, char *word_out)
 {
 	char words_phonemes[N_WORD_PHONEMES]; // a word translated into phoneme codes
@@ -1340,6 +1276,11 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 		Word_EmbeddedCmd();
 	}
 
+	if (n_ph_list2 >= N_PHONEME_LIST-2) {
+		// No room, can't translate anything
+		return 0;
+	}
+
 	if ((word[0] == 0) || (word_flags & FLAG_DELETE_WORD)) {
 		// nothing to translate.  Add a dummy phoneme to carry any embedded commands
 		if (embedded_flag) {
@@ -1356,6 +1297,11 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 		return 0;
 	}
 
+	if (n_ph_list2 >= N_PHONEME_LIST-7-2) {
+		// We may require up to 7 phonemes, plus the 2 phonemes from the caller, can't translate safely
+		return 0;
+	}
+
 	// after a $pause word attribute, ignore a $pause attribute on the next two words
 	if (tr->prepause_timeout > 0)
 		tr->prepause_timeout--;
@@ -1487,7 +1433,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 
 				new_language = (char *)(&p[1]);
 				if (new_language[0] == 0)
-					new_language = "en";
+					new_language = ESPEAKNG_DEFAULT_VOICE;
 
 				switch_phonemes = SetTranslator2(new_language);
 
@@ -1551,7 +1497,10 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	if ((flags & FLAG_FOUND) && !(flags & FLAG_TEXTMODE))
 		found_dict_flag = SFLAG_DICTIONARY;
 
-	while ((pre_pause > 0) && (n_ph_list2 < N_PHONEME_LIST-4)) {
+	// Each iteration may require up to 1 phoneme
+	// and after this loop we may require up to 7 phonemes
+	// and our caller requires 2 phonemes
+	while ((pre_pause > 0) && (n_ph_list2 < N_PHONEME_LIST-7-2)) {
 		// add pause phonemes here. Either because of punctuation (brackets or quotes) in the
 		// text, or because the word is marked in the dictionary lookup as a conjunction
 		if (pre_pause > 1) {
@@ -1566,7 +1515,9 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 		tr->prev_dict_flags[1] = 0;
 	}
 	plist2 = &ph_list2[n_ph_list2];
+	// From here we may require up to 4+1+3 phonemes
 
+	// This may require up to 4 phonemes
 	if ((option_capitals == 1) && (word_flags & FLAG_FIRST_UPPER)) {
 		SetPlist2(&ph_list2[n_ph_list2++], phonPAUSE_SHORT);
 		SetPlist2(&ph_list2[n_ph_list2++], phonCAPITAL);
@@ -1577,6 +1528,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 		}
 	}
 
+	// This may require up to 1 phoneme
 	if (switch_phonemes >= 0) {
 		if ((p[0] == phonPAUSE) && (p[1] == phonSWITCH)) {
 			// the new word starts with a phoneme table switch, so there's no need to switch before it.
@@ -1605,7 +1557,10 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 		p[1] = 0;
 	}
 
-	while (((ph_code = *p++) != 0) && (n_ph_list2 < N_PHONEME_LIST-4)) {
+	// Each iteration may require up to 1 phoneme
+	// and after this loop we may require up to 3 phonemes
+	// and our caller requires 2 phonemes
+	while (((ph_code = *p++) != 0) && (n_ph_list2 < N_PHONEME_LIST-3-2)) {
 		if (ph_code == 255)
 			continue; // unknown phoneme
 
@@ -1697,7 +1652,9 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 			first_phoneme = false;
 		}
 	}
+	// From here, we may require up to 3 phonemes
 
+	// This may require up to 1 phoneme
 	if (word_flags & FLAG_COMMA_AFTER)
 		SetPlist2(&ph_list2[n_ph_list2++], phonPAUSE_CLAUSE);
 
@@ -1709,6 +1666,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	if ((stress >= 4) && (phoneme_tab[ph_list2[n_ph_list2-1].phcode]->type == phVOWEL))
 		tr->end_stressed_vowel = 1; // word ends with a stressed vowel
 
+	// This may require up to 1 phoneme
 	if (switch_phonemes >= 0) {
 		// this word uses a different phoneme table, now switch back
 		strcpy(dictionary_name, old_dictionary_name);
@@ -1718,6 +1676,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	}
 
 
+	// This may require up to 1 phoneme
 	if (pitch_raised > 0) {
 		embedded_list[embedded_ix++] = EMBED_P+0x60+0x80 + (pitch_raised << 8); // lower pitch
 		SetPlist2(&ph_list2[n_ph_list2], phonPAUSE_SHORT);
@@ -1917,7 +1876,7 @@ static int TranslateChar(Translator *tr, char *ptr, int prev_in, unsigned int c,
 			if (final > 0)
 				*insert = final + 0x11a7;
 		} else {
-			// extact the initial and insert the remainder with a null initial
+			// extract the initial and insert the remainder with a null initial
 			c = initial + 0x1100;
 			*insert = (11*28*21) + (medial*28) + final + 0xac00;
 		}
@@ -2124,7 +2083,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	}
 	words[0].length = k;
 
-	while (!finished && (ix < (int)sizeof(sbuf) - 1) && (n_ph_list2 < N_PHONEME_LIST-4)) {
+	while (!finished && (ix < (int)sizeof(sbuf) - 1)) {
 		prev_out2 = prev_out;
 		utf8_in2(&prev_out, &sbuf[ix-1], 1);
 
@@ -2155,7 +2114,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			c = ' ';
 		}
 
-		if ((c == CTRL_EMBEDDED) || (c == ctrl_embedded)) {
+		if (c == CTRL_EMBEDDED) {
 			// start of embedded command in the text
 			int srcix = source_index-1;
 
@@ -2317,8 +2276,8 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 				if (iswupper(c)) {
 					c = towlower2(c, tr);
 
-					if ((j = tr->langopts.param[LOPT_CAPS_IN_WORD]) > 0) {
-						if ((j == 2) && (syllable_marked == false)) {
+					if (tr->langopts.param[LOPT_CAPS_IN_WORD]) {
+						if (syllable_marked == false) {
 							char_inserted = c;
 							c = 0x2c8; // stress marker
 							syllable_marked = true;
@@ -2348,7 +2307,8 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 					}
 				} else {
 					if ((all_upper_case) && (letter_count > 2)) {
-						if ((c == 's') && (next_in == ' ')) {
+						// Flag as plural only English
+						if (tr->translator_name == L('e', 'n') && (c == 's') && (next_in == ' ')) {
 							c = ' ';
 							all_upper_case |= FLAG_HAS_PLURAL;
 
@@ -2399,7 +2359,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 						c = ' '; // remove the dot if it's followed by a space or hyphen, so that it's not pronounced
 				}
 			} else if (c == '\'') {
-				if (((prev_in == '.') || iswalnum(prev_in)) && IsAlpha(next_in)) {
+				if (((prev_in == '.' && next_in == 's') || iswalnum(prev_in)) && IsAlpha(next_in)) {
 					// between two letters, or in an abbreviation (eg. u.s.a.'s). Consider the apostrophe as part of the word
 					single_quoted = false;
 				} else if ((tr->langopts.param[LOPT_APOSTROPHE] & 1) && IsAlpha(next_in))
@@ -2551,7 +2511,9 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	}
 	words[0].flags |= FLAG_FIRST_WORD;
 
-	for (ix = 0; ix < word_count; ix++) {
+	// Each TranslateWord2 may require up to 7 phonemes
+	// and after this loop we require 2 phonemes
+	for (ix = 0; ix < word_count && (n_ph_list2 < N_PHONEME_LIST-7-2); ix++) {
 		int nx;
 		int c_temp;
 		char *pn;
@@ -2602,7 +2564,9 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 		if (n_digits > 4) {
 			// word is entirely digits, insert commas and break into 3 digit "words"
 			number_buf[0] = ' ';
-			pn = &number_buf[1];
+			number_buf[1] = ' ';
+			number_buf[2] = ' ';
+			pn = &number_buf[3];
 			nx = n_digits;
 			nw = 0;
 
@@ -2646,7 +2610,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 			pn[16] = 0;
 			nw = 0;
 
-			for (pw = &number_buf[1]; pw < pn;) {
+			for (pw = &number_buf[3]; pw < pn;) {
 				// keep wflags for each part, for FLAG_HYPHEN_AFTER
 				dict_flags = TranslateWord2(tr, pw, &num_wtab[nw++], words[ix].pre_pause);
 				while (*pw++ != ' ')
@@ -2712,7 +2676,7 @@ void TranslateClause(Translator *tr, int *tone_out, char **voice_change)
 	if (Eof() && ((word_count == 0) || (option_endpause == 0)))
 		clause_pause = 10;
 
-	MakePhonemeList(tr, clause_pause, new_sentence2, &n_ph_list2, ph_list2);
+	MakePhonemeList(tr, clause_pause, new_sentence2);
 	phoneme_list[N_PHONEME_LIST].ph = NULL; // recognize end of phoneme_list array, in Generate()
 	phoneme_list[N_PHONEME_LIST].sourceix = 1;
 
